/***************************************************************************
 *   Copyright (C) 2007 by GNB	(Grupo Computacion Biologica), EPS (Madrid)*
 *   Carlos Muñiz Solaz							   * 
 *   carlos.muniz@uam.es   						   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//  /***************************************************************************
//                            LibraryDialog.cpp  -  description
//                               -------------------
//      begin                : Sat Aug 19 2000
//      copyright            : (C) 2000, 2001 by llornkcor
//      email                : ljp@llornkcor.com
//                            ***************************************************/
//  /***************************************************************************
//   *                                                                         *
//   *   This program is free software; you can redistribute it and/or modify  *
//   *   it under the terms of the GNU General Public License as published by  *
//   *   the Free Software Foundation; either version 2 of the License, or     *
//   *   (at your option) any later version.                                   *
//   *                                                                         *
//   ***************************************************************************/
/* CConfigFile.cpp */

#include "CConfigFile.h"
#define MAX_LINE_LENGTH 1500

CConfigFile::CConfigFile (const QString & filename)
{
    setName (filename);

    // create default group
    setGroup ("Default");

    dataFile = NULL;
}

CConfigFile::CConfigFile ()
{
    dataFile = NULL;
}

void
CConfigFile::open ()
{
    if (fname.isEmpty ())
        return;

    // open file
    dataFile = new QFile (fname);

    dataFile->open (IO_Raw | IO_ReadWrite | IO_Append);
    return;
}				// End open

void
CConfigFile::close ()
{
    if (!dataFile)
        dataFile->close ();
}				// End close


CConfigFile::~CConfigFile ()
{
    clear ();
}

void
CConfigFile::clear ()
{
    ConfigMap::Iterator it;

    for (it = map.begin (); it != map.end (); ++it)
        delete it.data ();

    header = QStringList ();
}

void
CConfigFile::setName (const QString & filename)
{
    // assign file name
    fname = filename;
}

void
CConfigFile::setGroup (const QString & group)
{
    if (map.find (group) == map.end ())
    {
        // new group
        currentgroup = new CConfigGroup (group);
        CHECK_PTR (currentgroup);

        map.insert (currentgroup->name (), currentgroup);
    }
    else
        currentgroup = map[group];
}

QStringList & CConfigFile::getGroupKeys ()
{
    QStringList *
    tmp = new QStringList ();

    // write group data
    GroupMap::Iterator git;

    for (git = currentgroup->map.begin (); git != currentgroup->map.end ();
            ++git)
        tmp->append (git.key ());
    return (*tmp);
}

const
QString &
CConfigFile::group () const
{
    return currentgroup->name ();
}

void
CConfigFile::writeEntry (const QString & key, const QString & value)
{
    currentgroup->writeEntry (key, value);
}

void
CConfigFile::writeEntry (const QString & key, int value)
{
    QString tmp;
    tmp.setNum ((long) value);

    currentgroup->writeEntry (key, tmp);
}

void
CConfigFile::writeEntry (const QString & key, double value)
{
    QString tmp;
    tmp.setNum ((double) value);

    currentgroup->writeEntry (key, tmp);
}

void
CConfigFile::writeEntryToFile (const QString & key, const QString & value)
{
    dataFile->writeBlock (key + " = " + value + "\n",
                          key.length () + 3 + value.length () + 1);


}

void
CConfigFile::writeEntryToFile (const QString & key, int value)
{
    if (!dataFile)
        return;

    dataFile->writeBlock (key + " = " + QString::number (value) + "\n",
                          key.length () + 3 +
                          QString::number (value).length () + 1);

}

void
CConfigFile::writeEntryToFile (const QString & key, double value)
{
    if (!dataFile)
        return;

    dataFile->writeBlock (key + " = " + QString::number (value) + "\n",
                          key.length () + 3 +
                          QString::number (value).length () + 1);
}

void
CConfigFile::writeStringToFile (const QString & string)
{
    if (!dataFile)
        return;

    dataFile->writeBlock (string + "\n", string.length () + 1);
}


void
CConfigFile::writeCommentToFile (const QString & string)
{
    if (!dataFile)
        return;

    dataFile->writeBlock ("#-- " + string + "\n", 4 + string.length () + 1);
}

void
CConfigFile::writeOrnamentCommentToFile ()
{
    if (!dataFile)
        return;

    QString tmp =
        "#----------------------------------------------------------------#\n";

    dataFile->writeBlock (tmp, tmp.length ());
}

void
CConfigFile::writeOrnamentHeaderToFile ()
{
    if (!dataFile)
        return;

    QString tmp =
        "//////////////////////////////////////////////////////////////////\n";

    dataFile->writeBlock (tmp, tmp.length ());
}

void
CConfigFile::writeHeaderToFile (const QString & string)
{
    if (!dataFile)
        return;

    dataFile->writeBlock ("//" + string + "\n", 2 + string.length () + 1);
}


void
CConfigFile::writeHeaderToFile (const QString & key, double value)
{
    if (!dataFile)
        return;

    dataFile->writeBlock ("//" + key + " = " + QString::number (value) + "\n",
                          2 + key.length () +
                          QString::number (value).length () + 3 + 1);
}

void
CConfigFile::writeHeaderToFile (const QString & key, const QString & value)
{
    if (!dataFile)
        return;

    dataFile->writeBlock ("//" + key + " = " + value + "\n",
                          2 + key.length () + value.length () + 3 + 1);
}

void
CConfigFile::writeHeaderToFile (const QString & key, int value)
{
    if (!dataFile)
        return;

    dataFile->writeBlock (key + " = " + QString::number (value) + "\n",
                          key.length () + 3 +
                          QString::number (value).length () + 1);

}


void
CConfigFile::writeGroupToFile (const QString & groupName)
{
    if (!dataFile)
        return;

    // write group name
    dataFile->writeBlock ("\n[" + groupName + "]\n", 4 + groupName.length ());
}

void
CConfigFile::writeHeaderEntry (const QString & key, const QString & value)
{
    header.append (key + ": " + value);
}

void
CConfigFile::writeHeaderEntry (const QString & key, int value)
{
    header.append (key + ":" + QString::number (value));
}


void
CConfigFile::writeHeaderEntry (const QString & key, double value)
{
    header.append (key + ":" + QString::number (value));
}



const QString &
CConfigFile::readEntry (const QString & key, const QString & default_value)
{
    return currentgroup->readEntry (key, default_value);
}

int
CConfigFile::readEntry (const QString & key, int default_value)
{
    QString tmp;

    tmp = currentgroup->readEntry (key);
    if (tmp == QString::null)
        return default_value;
    else
        return tmp.toInt ();
}

double
CConfigFile::readEntry (const QString & key, double default_value)
{
    QString tmp;

    tmp = currentgroup->readEntry (key);
    if (tmp == QString::null)
        return default_value;
    else
        return tmp.toDouble ();
}

void
CConfigFile::write ()
{
    if (fname.isEmpty ())
        return;

    // write to file
    QFile file (name ());
    file.open (IO_WriteOnly);

    //<! Write Header
    QTextStream stream (&file);

    stream << "////////////////////////////////////////////////////////////" <<
    endl;
    QStringList::iterator end = header.end ();
    for (QStringList::iterator it = header.begin (); it != header.end (); ++it)
        stream << "// " << (*it) << endl;
    stream << "////////////////////////////////////////////////////////////" <<
    endl;

    stream << endl;

    ConfigMap::Iterator it;

    for (it = map.begin (); it != map.end (); ++it)
        if (!it.data ()->isEmpty ())
        {
            // write group name
            file.writeBlock ("[" + it.key () + "]\n", 3 + it.key ().length ());

            // write group data
            GroupMap::Iterator git;

            for (git = it.data ()->map.begin (); git != it.data ()->map.end ();
                    ++git)
                file.writeBlock (git.key () + " = " + git.data () + "\n",
                                 git.key ().length () + 3 + git.data ().length () +
                                 1);

            stream << endl;
        }

    file.close ();
}

bool
CConfigFile::read ()
{
    if (fname.isEmpty ())
        return false;

    // open file
    QFile file (name ());
    bool ret = file.open (IO_ReadOnly);
    if (ret == false)
        return false;

    QString line;

    while (file.readLine (line, MAX_LINE_LENGTH) != -1)
    {
        // is it a group name?
        int index = line.find ("[");

        if (index != -1)
        {
            // get group name
            int index2 = line.findRev ("]");

            setGroup (line.mid (index + 1, index2 - index - 1));
        }

        // is it a key = value string
        if (index == -1)
        {
            // simplify white space
            line = line.simplifyWhiteSpace ();
            index = line.find ("=");

            if (index != -1)
            {
                // get key and value
                QString key = line.left (index - 1);
                QString value = line.right (line.length () - (index + 2));

                writeEntry (key, value);
            }
        }
    }

    file.close ();

    // switch to default group
    setGroup ("Default");
    return true;
}

QStringList & CConfigFile::groupList ()
{
    QStringList *
    tmp = new QStringList ();

    ConfigMap::Iterator it;

    for (it = map.begin (); it != map.end (); ++it)
        tmp->append (it.key ());

    return (*tmp);
}




/*** CConfigGroup ***/
CConfigGroup::CConfigGroup (const QString & groupname)
{
    setName (groupname);
}

CConfigGroup::CConfigGroup ()
{
}

void
CConfigGroup::setName (const QString & gname)
{
    // assign group name
    groupname = gname;
}

void
CConfigGroup::writeEntry (const QString & key, const QString & value)
{
    // replace or insert new key
    map.replace (key, value);
}

const QString &
CConfigGroup::readEntry (const QString & key, const QString & vdefault)
{
    // return value, if exists or default value
    const QString & tmp = map[key];

    return (tmp.isEmpty ()? vdefault : tmp);
}
