/***************************************************************************
 *   Copyright (C) 2007 by GNB	(Grupo Computacion Biologica), EPS (Madrid)*
 *   Carlos Muñiz Solaz							   * 
 *   carlos.muniz@uam.es   						   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//  /***************************************************************************
//                            LibraryDialog.cpp  -  description
//                               -------------------
//      begin                : Sat Aug 19 2000
//      copyright            : (C) 2000, 2001 by llornkcor
//      email                : ljp@llornkcor.com
//                            ***************************************************/
//  /***************************************************************************
//   *                                                                         *
//   *   This program is free software; you can redistribute it and/or modify  *
//   *   it under the terms of the GNU General Public License as published by  *
//   *   the Free Software Foundation; either version 2 of the License, or     *
//   *   (at your option) any later version.                                   *
//   *                                                                         *
//   ***************************************************************************/
/********************************

  Simple config file class.

  Author: Ulrich Clanget
  Copying: Freeware

********************************/

/* CConfigGroup.h */

#ifndef CCONFIGFILE_H
#define CCONFIGFILE_H

// qt include
#include <qstring.h>
#include <qmap.h>
#include <qstringlist.h>
#include <qfile.h>
#include <qregexp.h>
#include <qvaluevector.h>

#include <iostream>
using namespace std;
#include <fstream>

// group map type
typedef QMap < QString, QString > GroupMap;


class CConfigGroup
{
    friend class CConfigFile;

public:

    CConfigGroup (const QString & groupname);
    CConfigGroup ();

    void setName (const QString & groupname);
    const QString & name () const
    {
        return groupname;
    }

    const QString & readEntry (const QString & key,
                               const QString & default_value = QString::null);
    void writeEntry (const QString & key, const QString & value);

    bool isEmpty () const
    {
        return map.isEmpty ();
    }

private:
    QString groupname;
    GroupMap map;
};


/* CConfigFile.h */

typedef QMap < QString, CConfigGroup * >ConfigMap;

class CConfigFile
{
public:
    CConfigFile ();
    CConfigFile (const QString & filename);
    ~CConfigFile ();
    void open ();
    void close ();
    // clean up
    void clear ();

    // file
    void setName (const QString & filename);
    const QString & name () const
    {
        return fname;
    }
    void write ();
    bool read ();

    // current group
    void setGroup (const QString & group);
    const QString & group () const;

    // group info
    unsigned int numGroups () const
    {
        return map.count ();
    }
    QStringList & groupList ();
    QStringList & getGroupKeys ();

    // write entries
    void writeEntry (const QString & key, const QString & value);
    void writeEntry (const QString & key, int value);
    void writeEntry (const QString & key, double value);

    //!< Write Header Entries
    void writeHeaderEntry (const QString & key, const QString & value);
    void writeHeaderEntry (const QString & key, int value);
    void writeHeaderEntry (const QString & key, double value);

    void writeEntryToFile (const QString & key, const QString & value);
    void writeEntryToFile (const QString & key, int value);
    void writeEntryToFile (const QString & key, double value);
    void writeStringToFile (const QString & string);
    void writeCommentToFile (const QString & string);
    void writeOrnamentCommentToFile ();
    void writeOrnamentHeaderToFile ();
    void writeHeaderToFile (const QString & string);
    void writeHeaderToFile (const QString & key, double value);
    void writeHeaderToFile (const QString & key, const QString & value);
    void writeHeaderToFile (const QString & key, int value);
    void writeGroupToFile (const QString & groupName);

    // read entries
    const QString & readEntry (const QString & key,
                               const QString & default_value);
    int readEntry (const QString & key, int default_value);
    double readEntry (const QString & key, double default_value);


private:
    QFile * dataFile;
    QString fname;
    ConfigMap map;
    CConfigGroup *currentgroup;
    QStringList header;
};

#endif
