/***************************************************************************
 *   Copyright (C) 2007 by GNB	(Grupo Computacion Biologica), EPS (Madrid)*
 *   Carlos Muñiz Solaz							   * 
 *   carlos.muniz@uam.es   						   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ShmMotor.h"


#include <rtai_shm.h>
#include <rtai_nam2num.h>


#include <math.h>
#include "../include/motor_shm.h"

#include <iostream>
using namespace std;


ShmMotor* ShmMotor::pinstance = 0; // initialize pointer

ShmMotor *ShmMotor::Instance()
{
    if (pinstance == 0)
    {
        pinstance = new ShmMotor();

        if ( pinstance->init() == false)
        {
            pinstance = 0;
        }
    }

    return pinstance;
} 

bool ShmMotor::init ()
{

    //!< A pointer to the shared memory
    if (!(motor_shm = (MOTOR_SHM *) rtai_malloc(nam2num(MOTORSHM), sizeof(MOTOR_SHM))))
    {
        cout << "ERROR: Failed to connect with shared memory" << endl;
        return false;
    }

    return true;
}// End init

ShmMotor::ShmMotor( )
{
	// Empty
} 


ShmMotor::~ShmMotor ()
{
    if ( motor_shm != 0)
    {
        rtai_free (nam2num(MOTORSHM), motor_shm);
    }

    pinstance = 0;
} 

double ShmMotor::getSpeed ()
{
    return motor_shm->speed;
}

double ShmMotor::getRangle ()
{
    return motor_shm->right_angle;
}

double ShmMotor::getLangle ()
{
    return motor_shm->left_angle;
}

DIRECTION ShmMotor::getDirection ()
{
    return motor_shm->direction;
}

double ShmMotor::getPosition ()
{
    return motor_shm->curr_position;
}

double ShmMotor::getRightBound ()
{
    return motor_shm->bound_right;
}

double ShmMotor::getLeftBound ()
{
    return motor_shm->bound_left;
}

double ShmMotor::getPeriod ()
{
    return motor_shm->period;
}

double ShmMotor::getAcceleration()
{
    return motor_shm->acceleration;
}

double ShmMotor::getTimeAccel()
{
    return motor_shm->t_accel;
}

void ShmMotor::setSpeed (double speed)
{
    motor_shm->speed = speed;
}

void ShmMotor::setRangle (double right_angle)
{
    motor_shm->right_angle = right_angle;
}

void ShmMotor::setPosition (double position)
{
    motor_shm->curr_position = position;
}

void ShmMotor::setLangle (double left_angle)
{
    motor_shm->left_angle = left_angle;
}

void ShmMotor::setDirection (DIRECTION direction)
{
    motor_shm->direction = direction;
}

void ShmMotor::setPeriod (double period)
{
    motor_shm->period = period;
}

void ShmMotor::setAcceleration(double acceleration)
{
    motor_shm->acceleration = acceleration;
}

void ShmMotor::setTimeAccel(double t_accel)
{
    motor_shm->t_accel = t_accel;
}

void ShmMotor::calibrate ()
{
    motor_shm->state_flag = CALIBRATE;
}

void ShmMotor::moveRight(double angle)
{
    stopMotor();
    calibrate ();
    if (motor_shm->incremental == 1)
    {
        motor_shm->right_angle = STEP_ANGLE + motor_shm->curr_position;
        motor_shm->state_flag = RIGHT_MOV;
    }
    else
    {
        double curr_position = round (motor_shm->n_steps*0.15);

        motor_shm->right_angle = angle + curr_position;
        motor_shm->state_flag = RIGHT_MOV;
    }
}


void ShmMotor::moveLeft(double angle)
{
    stopMotor();
    calibrate ();

    if (motor_shm->incremental == 1 )
    {
        motor_shm->left_angle = motor_shm->curr_position - STEP_ANGLE;
        motor_shm->state_flag = LEFT_MOV;
    }
    else
    {
        double curr_position = round (motor_shm->n_steps * 0.15);

        motor_shm->left_angle = curr_position - angle;
        motor_shm->state_flag = LEFT_MOV;
    }
}

void ShmMotor::stopMotor()
{ 
    cout << __PRETTY_FUNCTION__ << endl;
    motor_shm->periodic = 0;
    motor_shm->forever = 0;
    motor_shm->state_flag = STOP_MOTOR;
}


void ShmMotor::changeSpeed(double speed)
{
    stopMotor();
    //calibrate();
    motor_shm->speed = speed;
    motor_shm->state_flag=CHANGE_SPEED;
}

void ShmMotor::accelerate(double accel, double TimeAccel, int direction)
{
    motor_shm->acceleration = accel;
    motor_shm->t_accel = TimeAccel;
    motor_shm->state_flag = ACCELERATE;
    if ( direction == 1)
        motor_shm->direction = RIGHT;
    if (direction == 2)
        motor_shm->direction = LEFT;

}

void ShmMotor::setRightBound (double rightBound)
{
    motor_shm->bound_right = rightBound;
}

void ShmMotor::setLeftBound (double leftBound)
{
    motor_shm->bound_left = leftBound;
}

void ShmMotor::setPeriodic (double finalTime, double angle_right, double angle_left)
{
    // Set the final time of the period and go to oscillate state for a while ;-)
    motor_shm->right_angle = angle_right + motor_shm->curr_position;
    motor_shm->left_angle = motor_shm->curr_position - angle_left;
    motor_shm->final_t = finalTime;
    motor_shm->state_flag = SET_PERIODIC;
}

void ShmMotor::setForever( double angle_right, double angle_left)
{
    motor_shm->right_angle = angle_right + motor_shm->curr_position;
    motor_shm->left_angle = motor_shm->curr_position - angle_left ;
    motor_shm->state_flag = FOREVER;
}

void ShmMotor::rewindMotor()
{
    motor_shm->state_flag = RESET_MOTOR;
}

void ShmMotor::setIncremental()
{
    motor_shm->state_flag = INC_MODE;
}

void ShmMotor::normalMode()
{
    motor_shm->incremental = 0;
}




/// Motor Model
void ShmMotor::setActiveModel(int active)
{
    motor_shm->active_model = active;
}

int ShmMotor::getActiveModel()
{
    return motor_shm->active_model;
}

void ShmMotor::setSpikeThreshold(double threshold)
{
    motor_shm->threshold = threshold;
}

double ShmMotor::getSpikeThreshold()
{
    return motor_shm->threshold;
}

void ShmMotor::setBurstRate (int rate)
{
    motor_shm->rate = rate;
}

int ShmMotor::getBurstRate()
{
    return motor_shm->rate;
}

void ShmMotor::setSpikeRefractoryPeriod(double period)
{
    motor_shm->spike_ref_period = period;
}

double ShmMotor::getSpikeRefractoryPeriod()
{
    return  motor_shm->spike_ref_period;
}

void ShmMotor::setBurstRefractoryPeriod(double period)
{
    motor_shm->burst_ref_period = period;
}

double ShmMotor::getBurstRefractoryPeriod()
{
    return motor_shm->burst_ref_period;
}

void ShmMotor::setBurstWindow(double window)
{
    motor_shm->burst_window = window;
}

double ShmMotor::getBurstWindow()
{
    return motor_shm->burst_window;
}

void ShmMotor::setProtocol (int protocol)
{
    motor_shm->protocol = protocol;
}

int ShmMotor::getProtocol()
{
    return motor_shm->protocol;
}

void ShmMotor::setInitDirection(int direction)
{
    // 0: Left, 1: Right
    motor_shm->init_direction = direction;
}

int ShmMotor::getInitDirection()
{
    return motor_shm->init_direction;
}















