/***************************************************************************
 *   Copyright (C) 2007 by GNB	(Grupo Computacion Biologica), EPS (Madrid)*
 *   Carlos Muñiz Solaz							   * 
 *   carlos.muniz@uam.es   						   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef __SHMMOTOR_H__
#define __SHMMOTOR_H__

using namespace std;
#include <iostream>


#include "../include/motor_shm.h"
#include <fcntl.h>



class ShmMotor
{

private:
    static ShmMotor* pinstance;

    MOTOR_SHM *motor_shm;
public:
    ~ShmMotor();
    bool init();

    // Instantiate the class
    static ShmMotor *Instance();

    double getSpeed ();
    double getRangle ();
    double getLangle ();
    double getRightBound ();
    double getLeftBound ();
    DIRECTION getDirection ();
    double getPosition ();
    double getPeriod ();
    double getAcceleration();
    double getTimeAccel();

    void setSpeed (double);
    void setRangle (double);
    void setLangle (double);
    void setDirection (DIRECTION direction);
    void setPosition (double);
    void setRightBound (double);
    void setLeftBound (double);
    void setPeriod (double);
    void setAcceleration(double acceleration);
    void setTimeAccel(double t_accel);
    void calibrate ();
    void moveRight(double angle);
    void moveLeft(double angle);
    void stopMotor();
    void changeSpeed(double speed);
    void accelerate(double accel, double TimeAccel, int direction);
    void setPeriodic (double finalTime, double angle_right, double angle_left);
    void setForever(double angle_right, double angle_left);
    void rewindMotor();
    void setIncremental();
    void normalMode();

    /// Motor Model
    void setActiveModel(int active);
    int getActiveModel();
    void setSpikeThreshold(double threshold);
    double getSpikeThreshold();
    void setBurstRate (int rate);
    int getBurstRate();
    void setSpikeRefractoryPeriod(double period);
    double getSpikeRefractoryPeriod();
    void setBurstRefractoryPeriod(double period);
    double getBurstRefractoryPeriod();
    void setBurstWindow(double window);
    double getBurstWindow();
    void setProtocol(int protocol);
    int getProtocol();
    void setInitDirection(int direction);
    int getInitDirection();

protected:
    ShmMotor();
};

#endif


