#ifndef __MOTOR_SHM_
#define __MOTOR_SHM_

#define MOTORSHM "motshm"

#define MAX_SPEED 30.0
#define MIN_SPEED 0.072
#define STEP_ANGLE 0.15

typedef enum { RIGHT, LEFT } DIRECTION;
typedef enum { CALIBRATE, RIGHT_MOV, LEFT_MOV, STOP_MOTOR, CHANGE_SPEED, RESET_MOTOR, SET_PERIODIC, INC_MODE, FOREVER, ACCELERATE } STATE;

typedef struct
{
    double speed;
    double right_angle;
    double left_angle;
    double bound_left;
    double bound_right;
    double acceleration;


    DIRECTION direction;
    STATE state_flag;
    double curr_position;
    double period;

    int periodic; // 1 indicates motor is oscillating. 0 indicates motor is nominal mode
    int forever;
    int incremental; // 1 indicates incremental mode. 0 indicates motor is nominal mode


    double initial_t; // Time when a periodic movement is started
    double final_t;	  // Time when a periodic movement should stop (period time)
    double t;

    /*Acceleration*/
    int accelerate; // 1 indicates accelerated mode. 0 indicates motor is nominal mode
    double t_accel;  //time when an accelerated movement should stop



    long T;		// Period

    double n_steps_right;
    double n_steps_left;
    double n_steps;


    /********************************/
    // Extra for motor model
    int active_model;
    double threshold;
    int rate;
    double spike_ref_period;
    double burst_ref_period;
    double burst_window;
    int polarity;

    int protocol;
    int init_direction;  // 0: Left, 1: Right
    void (*protocol_func)(double, double);

    /********************************/


}MOTOR_SHM;


#endif
